"""
Utilities, split out for easier testing & mocking.
"""

from django.conf import settings


def get_course_from_modulestore(course_id):
    """
    Get course from modulestore.
    For whatever reason, this has to be a runtime import or it breaks a bunch
    of things.
    """
    # pylint: disable=import-outside-toplevel
    from xmodule.modulestore.django import modulestore

    return modulestore().get_course(course_id)


def get_configured_translation_languages():
    """
    Get configured languages for AI translation from settings.

    Returns:
    - List of{ "code": ISO language code, "label": Human readable language name }
    """
    return [
        {"code": code, "label": label}
        for code, label in settings.AI_TRANSLATIONS_LANGUAGE_CONFIG.items()
    ]



"""
translatable_xblocks Django application initialization.
"""

import pkg_resources
from django.apps import AppConfig
from edx_django_utils.plugins.constants import PluginSettings, PluginURLs


class TranslatableXBlocksConfig(AppConfig):
    """
    Configuration for the translatable_xblocks Django application.
    """

    name = "translatable_xblocks"

    plugin_app = {
        PluginURLs.CONFIG: {
            "lms.djangoapp": {
                PluginURLs.NAMESPACE: "translatable_xblocks",
                PluginURLs.APP_NAME: "translatable_xblocks",
                PluginURLs.REGEX: r"^api/translatable_xblocks/",
                PluginURLs.RELATIVE_PATH: "api.urls",
            },
            "cms.djangoapp": {
                PluginURLs.NAMESPACE: "translatable_xblocks",
                PluginURLs.APP_NAME: "translatable_xblocks",
                PluginURLs.REGEX: r"^api/translatable_xblocks/",
                PluginURLs.RELATIVE_PATH: "api.urls",
            },
        },
        PluginSettings.CONFIG: {
            "lms.djangoapp": {
                "production": {
                    PluginSettings.RELATIVE_PATH: "settings.production",
                },
                "common": {
                    PluginSettings.RELATIVE_PATH: "settings.common",
                },
                "devstack": {
                    PluginSettings.RELATIVE_PATH: "settings.devstack",
                },
            },
            "cms.djangoapp": {
                "production": {
                    PluginSettings.RELATIVE_PATH: "settings.production",
                },
                "common": {
                    PluginSettings.RELATIVE_PATH: "settings.common",
                },
                "devstack": {
                    PluginSettings.RELATIVE_PATH: "settings.devstack",
                },
            },
        },
    }

    def ready(self):
        """
        Django startup code: swap edx-platform mapping for blocks with our translated versions.

        See https://discuss.openedx.org/t/override-default-edx-platform-xblock-mappings-without-forking/12441
        """
        # List of translated block types, for dismbiguating entry points
        translated_blocks = ["html", "problem", "video"]

        for block_type in translated_blocks:
            block_entry_points = list(
                pkg_resources.iter_entry_points("xblock.v1", block_type)
            )

            # If we have multiple entry points, ours gets precedence over the Open-edX copy
            if len(block_entry_points) > 1:
                dist = pkg_resources.get_distribution("Open-edX")
                del dist.get_entry_map("xblock.v1")[block_type]



"""
Translatable version of edx-platform/xmodule.capa_block
"""

# pylint:  disable=unnecessary-lambda-assignment

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from xblock.fields import Scope
from xmodule.capa_block import ProblemBlock as BaseProblemBlock

from translatable_xblocks.base import TranslatableXBlock
from translatable_xblocks.fields import TranslatableString, TranslatableXMLString

# Make '_' a no-op so we can scrape strings. Using lambda instead of
#  `django.utils.translation.gettext_noop` because Django cannot be imported in this file
_ = lambda text: text


try:
    FEATURES = getattr(settings, "FEATURES", {})
except ImproperlyConfigured:
    FEATURES = {}


class ProblemBlock(TranslatableXBlock, BaseProblemBlock):
    """Our version of the ProblemBlock with added translation logic."""

    display_name = TranslatableString(
        display_name=_("Display Name"),
        help=_("The display name for this component."),
        scope=Scope.settings,
        # it'd be nice to have a useful default but it screws up other things; so,
        # use display_name_with_default for those
        default=_("Blank Problem"),
    )

    data = TranslatableXMLString(
        help=_("XML data for the problem"),
        scope=Scope.content,
        enforce_type=FEATURES.get("ENABLE_XBLOCK_XML_VALIDATION", True),
        default="<problem></problem>",
    )
